using System;
using System.IO;
using System.Threading.Tasks;
using Microsoft.AspNetCore.Mvc;
using PhoneStore.Exceptions;
using PhoneStore.Mappers;
using PhoneStore.Models;
using PhoneStore.Services.Interfaces;
using PhoneStore.ViewModels.PhoneViewModels;

namespace PhoneStore.Controllers
{
    public class PhonesController : Controller
    {
        private readonly IPhoneService _phoneService;
        private readonly IBrandService _brandService;

        public PhonesController(
            MobileContext db,
            IPhoneService phoneService, IBrandService brandService)
        {
            _phoneService = phoneService;
            _brandService = brandService;
        }
       
        [HttpGet]
        public IActionResult Index(int? brandId)
        {
            try
            {
                return View(_phoneService.GetIndexViewModel(brandId));
            }
            catch (EntityNotFoundException e)
            {
                return RedirectToAction("Error", "Errors", new {statusCode = 777});
            }
            catch (Exception e)
            {
                return RedirectToAction("Error", "Errors", new {statusCode = 500});
            }
        }

        [HttpGet]
        public IActionResult Create()
        {
            try
            {
                return View(_phoneService.GetCreateViewModel());
            }
            catch (EntityNotFoundException e)
            {
                return RedirectToAction("Error", "Errors", new {statusCode = 777});
            }
            catch (Exception e)
            {
                return RedirectToAction("Error", "Errors", new {statusCode = 500});
            }
        }
        
        [HttpPost]
        public async Task<IActionResult> Create(PhoneCreateViewModel model)
        {
            try
            {
                if (ModelState.IsValid)
                {
                    await _phoneService.CreateAsync(model);

                    return RedirectToAction("Index");
                }

                model.Brands = _brandService.GetAll();
                return View("Create", model);
            }
            catch (EntityNotFoundException e)
            {
                return RedirectToAction("Error", "Errors", new {statusCode = 777});
            }
            catch (FileNotFoundException)
            {
                return RedirectToAction("Error", "Errors", new {statusCode = 666});
            }
            catch(Exception)
            {
                return RedirectToAction("Error", "Errors", new {statusCode = 500});
            }
        }

        [HttpGet]
        public IActionResult Delete(int phoneId)
        {
            try
            {
                var phone = _phoneService.GetById(phoneId);
                if (phone is null)
                    return RedirectToAction("Error", "Errors", new {statusCode = 777});
            
                return View(phone);
            }
            catch (EntityNotFoundException e)
            {
                return RedirectToAction("Error", "Errors", new {statusCode = 777});
            }
            catch (FileNotFoundException)
            {
                return RedirectToAction("Error", "Errors", new {statusCode = 666});
            }
            catch(Exception)
            {
                return RedirectToAction("Error", "Errors", new {statusCode = 500});
            }
        }
        
        [HttpPost]
        [ActionName("Delete")]
        public IActionResult Confirm(int phoneId)
        {
            try
            {
                //TODO: Добавить удаление файла изображения.
                var phone = _phoneService.GetById(phoneId);
                if (phone is null)
                    return RedirectToAction("Error", "Errors", new {statusCode = 777});
                _phoneService.Delete(phoneId);
                return RedirectToAction("Index");
            }
            catch (EntityNotFoundException e)
            {
                return RedirectToAction("Error", "Errors", new {statusCode = 777});
            }
            catch (FileNotFoundException)
            {
                return RedirectToAction("Error", "Errors", new {statusCode = 666});
            }
            catch(Exception)
            {
                return RedirectToAction("Error", "Errors", new {statusCode = 500});
            }
        }

        [HttpGet]
        public IActionResult Edit(int phoneId)
        {
            try
            {
                return View(_phoneService.GetEditViewModel(phoneId));
            }
            catch (EntityNotFoundException e)
            {
                return RedirectToAction("Error", "Errors", new {statusCode = 777});
            }
            catch (FileNotFoundException)
            {
                return RedirectToAction("Error", "Errors", new {statusCode = 666});
            }
            catch(Exception)
            {
                return RedirectToAction("Error", "Errors", new {statusCode = 500});
            }
        }
        
        [HttpPost]
        public IActionResult Edit(PhoneEditCreateViewModel model)
        {
            try
            {
                if (model is null)
                    return RedirectToAction("Error", "Errors", new {statusCode = 777});
                string imagePath = string.Empty;
                if (model.File is null)
                    imagePath = model.Image;
                _phoneService.Update(model.MapToPhone(imagePath));
                return RedirectToAction("Index");
            }
            catch (EntityNotFoundException e)
            {
                return RedirectToAction("Error", "Errors", new {statusCode = 777});
            }
            catch (FileNotFoundException)
            {
                return RedirectToAction("Error", "Errors", new {statusCode = 666});
            }
            catch(Exception)
            {
                return RedirectToAction("Error", "Errors", new {statusCode = 500});
            }
        }

        [HttpGet]
        public IActionResult About(int? phoneId)
        {
            try
            {
                if (!phoneId.HasValue) return RedirectToAction("Error", "Errors", new {statusCode = 777});
                var phoneViewModel = _phoneService.GetById(phoneId.Value);
                return View(phoneViewModel);
            }
            catch (EntityNotFoundException e)
            {
                return RedirectToAction("Error", "Errors", new {statusCode = 777});
            }
            catch (FileNotFoundException)
            {
                return RedirectToAction("Error", "Errors", new {statusCode = 666});
            }
            catch(Exception)
            {
                return RedirectToAction("Error", "Errors", new {statusCode = 500});
            }
        }
    }
}